@@ -1,4 +1,4 @@
-# $Id: Brick.pm 2244 2007-03-26 07:03:12Z comdog $
+# $Id: Brick.pm 2275 2007-05-10 19:43:45Z comdog $
 package Brick;
 use strict;
 
@@ -7,9 +7,12 @@ use vars qw($VERSION);
 
 use Carp qw( carp croak );
 use Data::Dumper;
+use UNIVERSAL qw(isa);
 
-$VERSION = '0.221_01';
-	#sprintf "0.%04d_01", q$Revision: 2244 $ =~ m/(\d+)/g;
+use Brick::Profile;
+
+$VERSION = '0.223';
+	#sprintf "0.%04d_01", q$Revision: 2275 $ =~ m/(\d+)/g;
 
 =head1 NAME
 
@@ -23,7 +26,7 @@ Brick - Complex business rule data validation
 		external_packages => [ qw(Foo::Validator Bar::Validator) ]
 		} );
 
-	my @profile = (
+	my $profile = Brick::Profile->new( $brick,
 		[ required  => sub { .... }    => $hash ],
 		[ optional  => optional_fields => $hash ],
 
@@ -32,7 +35,12 @@ Brick - Complex business rule data validation
 		[ outside   => ex_number       => $hash ],
 		);
 
-	my $results = $brick->apply( \@profile, \%input );
+	my %input_from_app = (
+		name => 'Joe Snuffy',
+		...
+		);
+		
+	my $results = $brick->apply( $profile, \%%input_from_app );
 
 =head1 DESCRIPTION
 
@@ -72,7 +80,7 @@ sub _load_external_packages
 	foreach my $package ( @packages )
 		{
 		eval "package $bucket_class; require $package; $package->import";
-		carp "Could not load $package: $@" if $@;
+		croak "Could not load $package: $@" if $@;
 		}
 
 	}
@@ -159,7 +167,7 @@ sub create_bucket
 	{
 	my( $brick, $profile ) = @_;
 
-	unless( 0 == $brick->lint( $profile || [] ) ) # zero but true!
+	unless( 0 == $brick->profile_class->lint( $profile || [] ) ) # zero but true!
 		{
 		croak "Bad profile for create_bucket! Perhaps you need to check it with lint"
 		};
@@ -245,12 +253,12 @@ sub add_validator_packages
 
 Based on the current instance, create another one just like it but not
 connected to it (in effect forking the instance). After the C<clone>
-you can change new instance without affecting the old one. This is handy
-in C<explain>, for instance, where I want a deep copy for a moment. At least
-I think I want a deep copy.
+you can change new instance without affecting the old one. This is
+handy in C<explain>, for instance, where I want a deep copy for a
+moment. At least I think I want a deep copy.
 
-That's the idea. Right now this just returns the instance. When not using
-a copy breaks, I'll fix that.
+That's the idea. Right now this just returns the same instance. When
+not using a copy breaks, I'll fix that.
 
 =cut
 
@@ -261,184 +269,20 @@ sub clone
 	$brick;
 	}
 
-=item lint
-
-Examine the profile and complain about irregularities in format. This
-only checks the format; it does not try to determine if the profile
-works or makes sense. It returns a hash whose key is the index of the
-profile element and whose value is an anonymous hash to indicate what
-had the error:
-
-	format  -   the element is an arrayref
-	name    -   the name is a scalar
-	method  -   is a code ref or can be found in the package
-					$brick->bucket_class returns
-	args    -   the last element is a hash reference
-
-If the profile is not an array reference, C<lint> immediately returns
-undef or the empty list. In scalar context, C<lint> returns 0 for
-format success and the number of errors (so true) for format failures.
-If there is a format error (e.g. an element is not an array ref), it
-immediately returns the number of errors up to that point.
-
-	my $lint = $brick->lint( \@profile );
-
-	print do {
-		if( not defined $lint ) { "Profile must be an array ref\n" }
-		elsif( $lint )          { "Did not validate, had $lint problems" }
-		else                    { "Woo hoo! Everything's good!" }
-		};
-
-In list context, it returns a hash (a list of one element). The result
-will look something like this hash, which has keys for the elements
-that lint thinks are bad, and the values are anonymous hashes with
-keys for the parts that failed:
-
-	%lint = (
-		1 => {
-			method => "Could not find method foo in package",
-			},
-		4 => {
-			args => "Arguments should be a hash ref, but it was a scalar",
-			}
-		);
-
-If you are using C<AUTOLOAD> to generate some of the methods at
-runtime (i.e. after C<lint> has a chance to check for it), use a
-C<can> method to let C<lint> know that it will be available later.
-
-TO DO:
-
-Errors for duplicate names?
-
-=cut
-
-sub lint
-	{
-	my( $brick, $profile ) = @_;
-
-	return unless(
-		eval { $profile->isa( ref [] ) } or
-		UNIVERSAL::isa( $profile, ref [] )
-		);
-
-	my $lint = {};
-
-	foreach my $index ( 0 .. $#$profile )
-		{
-		my $h = $lint->{$index} = {};
-
-		unless( eval { $profile->[$index]->isa( ref [] ) } or
-			UNIVERSAL::isa(  $profile->[$index], ref [] )
-			)
-			{
-			$h->{format} = "Not an array reference!";
-			last;
-			}
-
-		my( $name, $method, $args ) = @{ $profile->[$index] };
-
-		$h->{name} = "Profile name is not a simple scalar!" if ref $name;
-
-		$h->{args} = "Couldn't find method [$method]" unless
-			eval { $method->isa( ref sub {} ) } or
-			UNIVERSAL::isa( $method, sub {} )    or
-			eval { $brick->bucket_class->can( $method ) }; #XXX which class though?
-
-		$h->{args} = "Args is not a hash reference" unless
-			eval { $args->isa( ref {} ) } or
-			UNIVERSAL::isa( $args, ref {} );
-
-		# args needs what?
-
-		delete $lint->{$index} if 0 == keys %{$lint->{$index}};
-		}
-
-#print STDERR Data::Dumper->Dump( [$lint], [qw(lint)] );
-#use Data::Dumper;
-
-	# set errors;
-	wantarray ? %$lint : ( scalar keys %$lint );
-	}
-
-=item explain( PROFILE_ARRAYREF )
-
-Turn the profile into a textual description without applying it to any
-data. This does not add the profile to instance and it does not add
-the constraints to the bucket.
-
-If everything goes right, this returns a single string that represents
-the profile.
-
-If the profile does not pass the C<lint> test, this returns undef or the
-empty list.
-
-If you want to do something with a datastructure, you probably want to
-write a different method very similar to this instead of trying to parse
-the output.
-
-Future notes: maybe this is just really a dispatcher to things that do
-it in different ways (text output, hash output).
-
-=cut
-
 sub explain
 	{
-	my( $brick, $profile ) = @_;
-
-	my $temp_bean = $brick->clone;
-
-	if( $temp_bean->lint( $profile ) )
-		{
-		carp "Profile did not validate!";
-		return;
-		}
-
-	my( $bucket, $refs ) = $temp_bean->create_bucket( $profile );
-		#print STDERR Data::Dumper->Dump( [ $bucket ], [qw(bucket)] );
-		#print STDERR Data::Dumper->Dump( [ $refs ], [qw(refs)] );
-
-	my @entries = map {
-		my $e = $bucket->get_from_bucket( $_ );
-		[ map { $e->$_ } qw(get_coderef get_name) ]
-		} @$refs;
-
-	#print STDERR Data::Dumper->Dump( [ \@entries ], [qw(entries)] );
-
-	my $level = 0;
-	my $str   = '';
-	foreach my $index ( 0 .. $#entries )
-		{
-		my $tuple = $entries[$index];
-
-		my @uses = ( [ $level, $tuple->[0] ] );
-
-		#print STDERR Data::Dumper->Dump( [ \@uses ], [qw(uses)] );
-
-		while( my $pair = shift @uses )
-			{
-			my $entry = $bucket->get_from_bucket( $pair->[1] );
-			#print Data::Dumper->Dump( [ $entry ], [qw(entry)] );
-			next unless $entry;
-
-			$str .=  "\t" x $pair->[0] . $entry->get_name . "\n";
-
-			unshift @uses, map {
-				[ $pair->[0] + 1, $_ ]
-				} @{ $entry->get_comprises( $pair->[1] ) };
-			#print Data::Dumper->Dump( [ \@uses ], [qw(uses)] );
-			}
-
-		$str.= "\n";
-		}
-
-	$str;
+	croak "Who's calling Brick::explain? That's in Brick::Profile now!";
 	}
 	
-=item apply(  PROFILE_ARRAYREF, INPUT_DATA_HASHREF )
+=item apply(  PROFILE OBJECT, INPUT_DATA_HASHREF )
 
-Apply the profile to the data in the input hash reference. It returns an
-array reference whose elements correspond to the elements in the profile.
+Apply the profile to the data in the input hash reference. The profile
+can either be a profile object or an array ref that apply() will use to
+create the profile object.
+
+This returns a results object blessed into the class name returned by
+results_class(), which is Brick::Result by default. If you don't like
+that, you can override it in your own subclass.
 
 =cut
 
@@ -446,62 +290,110 @@ sub apply
 	{
 	my( $brick, $profile, $input ) = @_;
 
-	my( $bucket, $refs ) = $brick->create_bucket( $profile );
-
+	croak "Did not get a profile object in Brick::apply()!\n" 
+		unless eval { $profile->isa( $brick->profile_class ) };
+	
+	my $bucket   = $profile->get_bucket;
+	my $coderefs = $profile->get_coderefs;
+	my $array    = $profile->get_array;
+	
 	my @entries = map {
 		my $e = $bucket->get_from_bucket( $_ );
 		[ map { $e->$_ } qw(get_coderef get_name) ]
-		} @$refs;
+		} @$coderefs;
 
 	my @results = ();
 
 	foreach my $index ( 0 .. $#entries )
 		{
 		my $e    = $entries[$index];
-		my $name = $profile->[$index][0];
-
-		print STDERR "Checking $name...\n" if $ENV{DEBUG};
-
+		my $name = $array->[$index][0];
+		
+		my $bucket_entry = $bucket->get_from_bucket( "$e->[0]" );
+		my $sub_name     = $bucket_entry->get_name;
+		
 		my $result = eval{ $e->[0]->( $input ) };
 		my $eval_error = $@;
+		
+		carp "Brick: $sub_name: eval error \$\@ is not a string or hash reference"
+			unless( ! ref $eval_error or UNIVERSAL::isa( $eval_error, ref {} ) );
 
-		#print STDERR Data::Dumper->Dump( [$eval_error], [qw(eval_error)] );
+		if( defined $eval_error and isa( $eval_error, ref {} ) )
+			{
+			$result = 0;
+			carp "Brick: $sub_name died with reference, but didn't define 'handler' key"
+				unless exists $eval_error->{handler};
+				
+			carp "Brick: $sub_name died with reference, but didn't define 'message' key"
+				unless exists $eval_error->{message};	
+			}
+		elsif( defined $eval_error ) # but not a reference
+			{
+			$eval_error = {
+				handler       => 'program_error',
+				message       => $eval_error,
+				program_error => 1,
+				errors        => [],
+				};
+			}
+			
+		my $handler = $array->[$index][1];
 
-		$result = 0 if ref $eval_error;
-
-		my $handler = $profile->[$index][1];
-
-		push @results, [ $name, $handler, $result, $@ ];
-		}
-
-	eval { eval "use " . $brick->result_class };
-	if( $@ ) 
-		{ 
-		carp "Couldn not load Result class " . $brick->result_class . "\n" 
+		my $result_item = $brick->result_class->result_item_class->new(
+			label    => $name,
+			method   => $handler,
+			result   => $result,
+			messages => $eval_error,
+			);
+			
+		push @results, $result_item;
 		}
 	
 	return bless \@results, $brick->result_class;
 	}
-	
+		
 =item bucket_class
 
-The namespace where the constraint building blocks are defined. By default
-this is C<Brick::Bucket>. If you don't like that, override this in a
-subclass.
+The namespace where the constraint building blocks are defined. By
+default this is C<Brick::Bucket>. If you don't like that, override
+this in a subclass. Things that need to work with the bucket class
+name, such as a factory method, will use the return value of this
+method.
+
+This method also loads the right class, so if you override it, 
+remember to load the class too!
 
 =cut
 
-sub bucket_class { 'Brick::Bucket' }
+sub bucket_class { require Brick::Bucket; 'Brick::Bucket' }
 
 =item result_class
 
 The namespace that C<apply> uses for its result object. By default
 this is C<Brick::Result>. If you don't like that, override this in a
-subclass.
+subclass. Things that need to work with the result class name, such as
+a factory method, will use the return value of this method.
+
+This method also loads the right class, so if you override it, 
+remember to load the class too!
 
 =cut
 
-sub result_class { 'Brick::Result' }
+sub result_class { require Brick::Result; 'Brick::Result' }
+
+=item profile_class
+
+The namespace for the profile object. By default this is
+C<Brick::Profile>. If you don't like that, override this in a
+subclass. Things that need to work with the result class name, such as
+a factory method, will use the return value of this method.
+
+This method also loads the right class, so if you override it, 
+remember to load the class too!
+
+=cut
+
+sub profile_class { require Brick::Profile; 'Brick::Profile' }
 
 =back
 
