@@ -1,9 +1,9 @@
-# $Id: Composers.pm 2243 2007-03-26 06:55:17Z comdog $
+# $Id: Composers.pm 2270 2007-05-09 20:21:21Z comdog $
 package Brick::Composers;
 use base qw(Exporter);
 use vars qw($VERSION);
 
-$VERSION = sprintf "1.%04d", q$Revision: 2243 $ =~ m/ (\d+) /xg;
+$VERSION = sprintf "1.%04d", q$Revision: 2270 $ =~ m/ (\d+) /xg;
 
 use Brick::Bucket;
 
@@ -204,15 +204,24 @@ sub __compose_not
 	}
 
 
+=item __compose_until_pass
 
 =item __compose_pass_or_skip
 
-Go through the list of closures, trying each one until one suceeds. If a closure
-doesn't suceed, don't fail, just move on. Return true for the first one that
-passes, short-circuited the rest. If none of the closures pass, die with an
-error noting that nothing passed.
+Go through the list of closures, trying each one until one suceeds. Once
+something succeeds, it returns the name of the subroutine that passed.
 
-This can still die for programming (not logic) errors.
+If
+a closure doesn't die, but doesn't return true, this doesn't fail but
+just moves on. Return true for the first one that passes,
+short-circuited the rest. 
+
+If none of the closures pass (and none of them die), return 0. This might
+be the odd case of a several selectors (see L<Brick::Selector>), none of 
+which pass.
+
+If one of the subs dies, this composer still dies. This can also die
+for programming (not logic) errors.
 
 =cut
 
@@ -228,8 +237,6 @@ sub __compose_pass_or_skip
 
 	my @caller = $bucket->__caller_chain_as_list();
 
-	my $max = @subs;
-
 	my $sub = $bucket->add_to_bucket( {
 		code => sub {
 			my $count = 0;
@@ -238,22 +245,20 @@ sub __compose_pass_or_skip
 			foreach my $sub ( @subs )
 				{
 				my $result = eval { $sub->( @_ ) };
-				my $at = $@;
-				#print STDERR "\tskip: Returned result: $result\n";
-				#print STDERR "\tskip: Returned undef!\n" unless defined $result;
-				#print STDERR "\tskip: Returned ref!\n" if ref $at;
+				my $eval_error = $@;
+
+				# all true values are success
 				return "$sub" if $result;   # we know we passed
-				return if ( ! defined $result and ! defined $at );  # we're a selector: failed with no error
-				#print STDERR "skip: Still going!\n";
+				
+				
+				# we're a selector: failed with no error
+				return if ( ! defined $result and ! defined $eval_error );  
 
-				die if( ref $at and $at );  # die for everything else
-				};
-
-			die {
-				message => "Nothing worked! Unexpected failure of all branches",
-				handler => $caller[0]{'sub'},
-				errors  => \@dies,
+				# die for everything else - validation error
+				die if( ref $eval_error );  
 				};
+				
+			return 0;
 			},
 		});
 
@@ -262,6 +267,12 @@ sub __compose_pass_or_skip
 	return $sub;
 	}
 
+BEGIN {
+*__compose_until_pass = *__compose_pass_or_skip;
+}
+
+=item __compose_until_fail
+
 =item __compose_pass_or_stop
 
 Keep going as long as the closures return true.
@@ -339,6 +350,10 @@ sub __compose_pass_or_stop
 	return $sub;
 	}
 
+BEGIN {
+*__compose_until_fail = *__compose_pass_or_stop;
+}
+
 =back
 
 =head1 TO DO
