@@ -4,7 +4,8 @@ use strict;
 
 use vars qw($VERSION);
 
-use Carp qw(croak);
+use Carp qw(carp croak);
+use UNIVERSAL qw(isa);
 
 $VERSION = sprintf "1.%04d", q$Revision: 2183 $ =~ m/ (\d+) /xg;
 
@@ -25,6 +26,24 @@ Brick::Result - the result of applying a profile
 This class provides methods to turn the data structure returned
 by apply() into a useable form for particular situations.
 
+
+=head2 Class methods
+
+=over 4
+
+=item result_item_class
+
+Loads and returns the class name to use for the elements of the Results
+data structure.
+
+=cut
+
+sub result_item_class { require Brick::Result::Item; 'Brick::Result::Item' };
+
+=back
+
+=head2 Instance methods
+
 =over
 
 =item explain
@@ -33,49 +52,64 @@ Create a string the shows the result in an outline form.
 
 =cut
 
+
+# for the $pair thing in explain
+use constant LEVEL   => 0;
+use constant MESSAGE => 1;
+
+
 sub explain
 	{
-	my( $result ) = @_;
+	my( $result_set ) = @_;
 
 	my $str   = '';
 
-	foreach my $element ( @$result )
-		{
+	foreach my $element ( @$result_set )
+		{		
 		my $level = 0;
 		
-		$str .= "$$element[0]: ";
+		$str .= "$$element[0]: " . do {
+			if( $element->passed )                  { "passed " }
+			elsif( $element->is_validation_error )  { "failed " }
+			elsif( $element->is_code_error )        { "code error in " }
+			};
+			
+		$str .= $element->get_method() . "\n";
 		
-		if( $element->[2] ) 
+		if( $element->passed )
 			{
-			$str .= "passed $$element[1]\n\n";
+			$str .= "\n";
 			next;
 			}
-		else
-			{
-			$str .= "failed $$element[1]\n";
-			}
-			
-		my @uses = ( [ $level, $element->[3] ] );
+		
+		# this descends into the error tree (without using recursion
+		my @uses = ( [ $level, $element->get_messages ] );
 
 		while( my $pair = shift @uses )
 			{
 			# is it a single error or a composition?
-			unless( ref $pair->[1] )
+			if( ! ref $pair->[ MESSAGE ] )
+				{
+				$str .= $pair->[ MESSAGE ] . "foo";
+				}
+			elsif( ! UNIVERSAL::isa( $pair->[ MESSAGE ], ref {} ) )
 				{
 				next;
 				}
-			elsif( exists $pair->[1]->{errors} )
+			elsif( exists $pair->[ MESSAGE ]->{errors} )
 				{
+				# something else to process, but put it back into @uses
 				unshift @uses, map {
-					[ $pair->[0] + 1, $_ ]
-					} @{ $pair->[1]->{errors} };
+					[ $pair->[ LEVEL ] + 1, $_ ]
+					} @{ $pair->[ MESSAGE ]->{errors} };
 				}
 			else
 				{
 				# this could come back as an array ref instead of a string
-				$str .=  "\t" . #x $pair->[0] . 
-					
-					join( ": ", @{ $pair->[1] }{qw(failed_field handler message)} ) . "\n";
+				no warnings 'uninitialized';
+				$str .=  "\t" . #x $pair->[ LEVEL ] . 
+					join( ": ", @{ $pair->[ MESSAGE ] 
+						}{qw(failed_field handler message)} ) . "\n";
 				}
 
 			}
@@ -94,21 +128,28 @@ Collapse the result structure to an array of flat hashes.
 
 sub flatten
 	{
-	my( $result ) = @_;
+	my( $result_set ) = @_;
 
 	my $str   = '';
 
 	my @flatten;
 	
-	foreach my $element ( @$result ) # one element per profile element
+	foreach my $element ( @$result_set ) # one element per profile element
 		{
-		next if $element->[2];
-		my $constraint = $element->[1];
+		bless $element, $result_set->result_item_class;
+		next if $element->passed;
+		my $constraint = $element->get_method;
 		
-		my @uses = ( $element->[3]);
+		my @uses = ( $element->get_messages );
 
 		while( my $hash = shift @uses )
 			{
+			if( ! isa $hash, ref {} )
+				{
+				carp "Non-hash reference in messages result key! Skipping";
+				next;
+				}
+				
 			# is it a single error or a composition?
 			unless( ref $hash  )
 				{
@@ -138,19 +179,19 @@ Similar to flatten, but keyed by the field that failed the constraint.
 
 sub flatten_by_field
 	{
-	my( $result ) = @_;
+	my( $result_set ) = @_;
 
 	my $str   = '';
 
 	my %flatten;
 	my %Seen;
 	
-	foreach my $element ( @$result ) # one element per profile element
+	foreach my $element ( @$result_set ) # one element per profile element
 		{
-		next if $element->[2];
-		my $constraint = $element->[1];
+		next if $element->passed;
+		my $constraint = $element->get_method;
 		
-		my @uses = ( $element->[3] );
+		my @uses = ( $element->get_messages );
 
 		while( my $hash = shift @uses )
 			{
@@ -166,7 +207,7 @@ sub flatten_by_field
 			else
 				{
 				my $field = $hash->{failed_field};
-				next if $Seen{$field}{$hash->{handler}}++;
+				next if $hash->{handler} and $Seen{$field}{$hash->{handler}}++;				
 				$flatten{ $field } = [] unless exists $flatten{ $field };
 				push @{ $flatten{ $field } }, 
 					{ %$hash, constraint => $constraint };
@@ -188,19 +229,19 @@ Similar to flatten, but keyed by the hash key named in the argument list.
 
 sub flatten_by
 	{
-	my( $result, $key ) = @_;
+	my( $result_set, $key ) = @_;
 
 	my $str   = '';
 
 	my %flatten;
 	my %Seen;
 	
-	foreach my $element ( @$result ) # one element per profile element
+	foreach my $element ( @$result_set ) # one element per profile element
 		{
-		next if $element->[2];
-		my $constraint = $element->[1];
+		next if $element->passed;
+		my $constraint = $element->get_method;
 		
-		my @uses = ( $element->[3] );
+		my @uses = ( $element->get_messages );
 
 		while( my $hash = shift @uses )
 			{
@@ -216,7 +257,7 @@ sub flatten_by
 			else
 				{
 				my $field = $hash->{$key};
-				next if $Seen{$field}{$hash->{handler}}++;
+				next if $hash->{handler} and $Seen{$field}{$hash->{handler}}++;				
 				$flatten{ $field } = [] unless exists $flatten{ $field };
 				push @{ $flatten{ $field } }, 
 					{ %$hash, constraint => $constraint };
@@ -238,8 +279,6 @@ What should this do?
 
 sub dump { croak "Not yet implemented" }
 
-
-
 =back
 
 =head1 TO DO
@@ -248,6 +287,7 @@ TBA
 
 =head1 SEE ALSO
 
+L<Brick::Tutorial>, L<Brick::UserGuide>
 
 =head1 SOURCE AVAILABILITY
 
